from __future__ import absolute_import

import datetime
import decimal

import pytest
from django.core.management import call_command

from .django_app.models import Rabbit, models, Hole, Door, Customer, Simple, Client, Tag, Message
from mixer.backend.django import Mixer


@pytest.fixture(autouse=True)
def mixer(request):
    call_command('migrate', interactive=False, verbosity=0)
    request.addfinalizer(lambda: call_command('flush', interactive=False, verbosity=0))
    return Mixer()


def test_base():
    from mixer.backend.django import mixer

    simple = mixer.blend('django_app.simple')
    assert isinstance(simple.value, int)


def test_fields(mixer):
    rabbit = mixer.blend('django_app.rabbit')

    assert isinstance(rabbit, Rabbit)
    assert rabbit.id
    assert rabbit.pk
    assert len(rabbit.title) <= 16
    assert isinstance(rabbit.active, bool)
    assert isinstance(rabbit.object_id, int)
    assert rabbit.object_id >= 0
    assert isinstance(rabbit.error_code, int)
    assert rabbit.error_code >= 0
    assert rabbit.error_code <= 70000
    assert isinstance(rabbit.created_at, datetime.date)
    assert isinstance(rabbit.updated_at, datetime.datetime)
    assert isinstance(rabbit.opened_at, datetime.time)
    assert '@' in rabbit.email
    assert isinstance(rabbit.speed, decimal.Decimal)
    assert rabbit.custom
    assert rabbit.text
    assert len(rabbit.text) <= 512
    assert rabbit.picture.read() == b'pylama\n'

    assert rabbit.ip.count('.') == 3
    for ip_section in rabbit.ip.split('.'):
        assert 0 <= int(ip_section) <= 255

    assert rabbit.ip6.count(':') == 7
    for ip_section in rabbit.ip6.split(':'):
        assert 0 <= int(ip_section, 16) <= 65535

    assert isinstance(rabbit.file_path, str)

    rabbit = mixer.blend('rabbit')
    assert rabbit


def test_random_fields():
    mixer = Mixer(fake=False)

    hat = mixer.blend('django_app.hat', color=mixer.RANDOM)
    assert hat.color in ('RD', 'GRN', 'BL')


def test_custom(mixer):
    mixer.register(
        Rabbit,
        title=lambda: 'Mr. Rabbit',
        speed=lambda: mixer.faker.small_positive_integer(99))

    rabbit = mixer.blend(Rabbit, speed=mixer.RANDOM, percent=23)
    assert isinstance(rabbit.speed, decimal.Decimal)
    assert isinstance(rabbit.percent, float)
    assert rabbit.title == 'Mr. Rabbit'

    from mixer.backend.django import GenFactory

    def getter(*args, **kwargs):
        return "Always same"

    class MyFactory(GenFactory):
        generators = {models.CharField: getter}

    fabric = MyFactory.get_fabric(models.CharField)
    assert fabric() == "Always same"

    mixer = Mixer(factory=MyFactory, fake=False)
    assert mixer._Mixer__factory == MyFactory

    test = mixer.blend(Rabbit)
    assert test.title == "Always same"

    @mixer.middleware('auth.user')
    def encrypt_password(user): # noqa
        user.set_password(user.password)
        return user

    user = mixer.blend('auth.User', password='test')
    assert user.check_password('test')

    user = user.__class__.objects.get(pk=user.pk)
    assert user.check_password('test')


def test_select(mixer):
    mixer.cycle(3).blend(Rabbit)
    hole = mixer.blend(Hole, rabbit=mixer.SELECT)
    assert not hole.rabbit

    rabbits = Rabbit.objects.all()
    hole = mixer.blend(Hole, owner=mixer.SELECT)
    assert hole.owner in rabbits

    rabbit = rabbits[0]
    hole = mixer.blend(Hole, owner=mixer.SELECT(email=rabbit.email))
    assert hole.owner == rabbit


def test_relation(mixer):
    hat = mixer.blend('django_app.hat')
    assert not hat.owner

    silk = mixer.blend('django_app.silk')
    assert not silk.hat.owner

    silk = mixer.blend('django_app.silk', hat__owner__title='booble')
    assert silk.hat.owner
    assert silk.hat.owner.title == 'booble'

    door = mixer.blend('django_app.door', hole__title='flash', hole__size=244)
    assert door.hole.owner
    assert door.hole.title == 'flash'
    assert door.hole.size == 244

    door = mixer.blend('django_app.door')
    assert door.hole.title != 'flash'

    num = mixer.blend('django_app.number', doors=[door])
    assert num.doors.get() == door

    num = mixer.blend('django_app.number')
    assert num.doors.count() == 0

    num = mixer.blend('django_app.number', doors__size=42)
    assert num.doors.all()[0].size == 42

    tag = mixer.blend('django_app.tag', customer=mixer.RANDOM)
    assert tag.customer


def test_many_to_many_through(mixer):
    pointa = mixer.blend('django_app.pointa', other=mixer.RANDOM)
    assert pointa.other.all()

    pointb = mixer.blend('pointb')
    pointa = mixer.blend('pointa', other=pointb)
    assert list(pointa.other.all()) == [pointb]


def test_many_to_many_random(mixer):
    mixer.cycle(5).blend('django_app.message')
    assert Message.objects.all()

    mixer.cycle(10).blend('django_app.tag', messages=mixer.RANDOM)
    assert Tag.objects.all()
    assert Tag.objects.all().count() == 10


@pytest.mark.skip(reason='Not implemented')
def test_many_to_many_select(mixer):
    mixer.cycle(5).blend('django_app.message')
    assert Message.objects.all()
    assert Message.objects.all().count() == 5

    mixer.cycle(10).blend('django_app.tag', messages=mixer.SELECT)
    assert Tag.objects.all()
    assert Message.objects.all().count() == 5


def test_random(mixer):
    user = mixer.blend(
        'auth.User', username=mixer.RANDOM('mixer', 'its', 'fun'))
    assert user.username in ('mixer', 'its', 'fun')

    rabbit = mixer.blend(Rabbit, url=mixer.RANDOM)
    assert '/' in rabbit.url


def test_mix(mixer):
    test = mixer.blend(Rabbit, title=mixer.MIX.username)
    assert test.title == test.username

    test = Rabbit.objects.get(pk=test.pk)
    assert test.title == test.username

    test = mixer.blend(Hole, title=mixer.MIX.owner.title)
    assert test.title == test.owner.title

    test = mixer.blend(Door, hole__title=mixer.MIX.owner.title)
    assert test.hole.title == test.hole.owner.title

    test = mixer.blend(Door, hole__title=mixer.MIX.owner.username(
        lambda t: t + 's hole'
    ))
    assert test.hole.owner.username in test.hole.title
    assert 's hole' in test.hole.title

    test = mixer.blend(Door, owner=mixer.MIX.hole.owner)
    assert test.owner == test.hole.owner


def test_contrib(mixer):
    from django.db import connection
    _ = connection.connection.total_changes
    assert mixer.blend('auth.user')
    assert connection.connection.total_changes - _ == 1

    _ = connection.connection.total_changes
    assert mixer.blend(Customer)
    assert connection.connection.total_changes - _ == 2


def test_invalid_scheme(mixer):
    with pytest.raises(ValueError):
        mixer.blend('django_app.Unknown')


def test_skip(mixer):
    rabbit = mixer.blend(Rabbit, created_at=mixer.SKIP, title=mixer.SKIP)
    assert rabbit.created_at
    assert not rabbit.title


def test_generic(mixer):
    rabbit = mixer.blend(Rabbit)
    assert rabbit.content_type
    assert rabbit.content_type.model_class()

    obj = mixer.blend(Simple)
    with mixer.ctx(loglevel='DEBUG'):
        rabbit = mixer.blend(Rabbit, content_object=obj)
    assert rabbit.content_object == obj
    assert rabbit.object_id == obj.pk
    assert rabbit.content_type.model_class() == Simple


def test_deffered(mixer):
    simples = mixer.cycle(3).blend(Simple)
    rabbits = mixer.cycle(3).blend(
        Rabbit, content_object=(s for s in simples)
    )
    assert rabbits

    rabbit = rabbits[0]
    rabbit = rabbit.__class__.objects.get(pk=rabbit.pk)
    assert rabbit.content_object


def test_unique(mixer):
    for _ in range(100):
        mixer.blend(Client)


def test_guard(mixer):
    r1 = mixer.guard(username='maxi').blend(Rabbit, username='maxi')
    r2 = mixer.guard(username='maxi').blend(Rabbit, username='maxi')
    assert r1
    assert r1 == r2


def test_reload(mixer):
    r1 = mixer.blend(Rabbit)
    r1.title = 'wrong title'
    r2 = mixer.reload(r1)
    assert r2 == r1
    assert r2.title != r1.title

    s1 = mixer.blend(Simple)
    r2, s2 = mixer.reload(r1, s1)
    assert s1 == s2


def test_small_positive_integer_field_not_too_large(mixer):
    """
    Django 1.10 doc spec for maximum allowable value
    Multiple assertions to account for sufficient randomization
    """
    for _ in range(4):
        rabbit = mixer.blend(Rabbit)
        assert rabbit.error_code <= 32767
        assert rabbit.error_code > 0



import marshmallow as ma
import datetime as dt
import pytest


class Person(ma.Schema):

    name = ma.fields.String()
    status = ma.fields.String(
        required=True,
        validate=ma.validate.OneOf(choices=('user', 'moderator', 'admin')))
    created = ma.fields.DateTime()
    birthday = ma.fields.Date()
    is_relative = ma.fields.Bool()


class Pet(ma.Schema):

    name = ma.fields.String()
    animal_type = ma.fields.String(default='cat')
    awards = ma.fields.List(ma.fields.Str)
    owner = ma.fields.Nested(Person, many=True)


@pytest.fixture
def mixer():
    from mixer.backend.marshmallow import Mixer
    return Mixer(required=True)


def test_mixer(mixer):
    person = mixer.blend(Person)
    assert person['name']
    assert person['created']
    assert isinstance(person['is_relative'], bool)
    assert person['status'] in ('user', 'moderator', 'admin')

    pet = mixer.blend(Pet)
    assert pet['name']
    assert pet['animal_type'] == 'cat'
    assert pet['awards'] is not None
    assert pet['owner']
    assert pet['owner'][0]['name']



import tempfile

TMPDIR = tempfile.mkdtemp()

ROOT_URLCONF = 'tests.django_app.urls'

SECRET_KEY = 'KeepMeSecret'

DEBUG = True

MEDIA_ROOT = TMPDIR

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': ':memory:',
    }
}

USE_TZ = True

MIDDLEWARE_CLASSES = (
    'django.middleware.common.CommonMiddleware', 'django.middleware.csrf.CsrfViewMiddleware')

INSTALLED_APPS = 'django.contrib.contenttypes', 'django.contrib.auth', 'tests.django_app'



import datetime as dt

import pytest
from peewee import *


db = SqliteDatabase(':memory:')


class Person(Model):
    name = CharField()
    status = CharField(choices=(('user', 'user'), ('moderator', 'moderator'), ('admin', 'admin')))
    created = DateTimeField(default=dt.datetime.now)
    birthday = DateField()
    is_relative = BooleanField()

    class Meta:
        database = db


class Pet(Model):
    owner = ForeignKeyField(Person, related_name='pets')
    name = CharField()
    animal_type = CharField()

    class Meta:
        database = db


Person.create_table()
Pet.create_table()


@pytest.fixture
def mixer():
    from mixer.backend.peewee import mixer
    return mixer


@pytest.fixture(autouse=True)
def clean_tables():
    Person.delete().execute()
    Pet.delete().execute()


def test_mixer(mixer):
    person = mixer.blend(Person)
    assert person.name
    assert person.id
    assert person.birthday
    assert person.status in ('user', 'moderator', 'admin')

    pet = mixer.blend(Pet)
    assert pet.name
    assert pet.animal_type
    assert pet.owner

    with mixer.ctx(commit=False):
        person = mixer.blend(Person)
        assert person.id


def test_guard(mixer):
    person = mixer.blend(Person)
    person2 = mixer.guard(Person.name == person.name).blend(Person)
    assert person.id == person2.id


def test_reload(mixer):
    person = mixer.blend(Person, name='true')
    person.name = 'wrong'

    person = mixer.reload(person)
    assert person.name == 'true'


def test_select(mixer):
    person = mixer.blend(Person)
    pet = mixer.blend(Pet, owner=mixer.SELECT)
    assert person == pet.owner



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
from tests.django_app.models import CustomField
from django.conf import settings


class Migration(migrations.Migration):

    dependencies = [
        ('auth', '0001_initial'),
        ('contenttypes', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='Client',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('username', models.CharField(max_length=20)),
                ('city', models.CharField(unique=True, max_length=20)),
                ('name', models.CharField(max_length=50)),
                ('created_at', models.DateField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('score', models.IntegerField(default=50)),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='Customer',
            fields=[
                ('user_ptr', models.OneToOneField(parent_link=True, auto_created=True, primary_key=True, serialize=False, to=settings.AUTH_USER_MODEL, on_delete=models.CASCADE)),
                ('name', models.CharField(max_length=100)),
            ],
            options={
                'abstract': False,
                'verbose_name': 'user',
                'verbose_name_plural': 'users',
            },
            bases=('auth.user',),
        ),
        migrations.CreateModel(
            name='Door',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('size', models.PositiveIntegerField()),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='Hat',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('color', models.CharField(max_length=50, choices=[(b'RD', b'red'), (b'GRN', b'green'), (b'BL', b'blue')])),
                ('brend', models.CharField(default=b'wood', max_length=10)),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='Hole',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('title', models.CharField(max_length=16)),
                ('size', models.SmallIntegerField()),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='Message',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('content', models.TextField()),
                ('client', models.ForeignKey(to='django_app.Client', on_delete=models.CASCADE)),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='Number',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='ColorNumber',
            fields=[
                ('number_ptr', models.OneToOneField(parent_link=True, auto_created=True, primary_key=True, serialize=False, to='django_app.Number', on_delete=models.CASCADE)),
                ('color', models.CharField(max_length=20)),
            ],
            options={
            },
            bases=('django_app.number',),
        ),
        migrations.CreateModel(
            name='PointA',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='PointB',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='Rabbit',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('title', models.CharField(max_length=16)),
                ('username', models.CharField(unique=True, max_length=16)),
                ('active', models.BooleanField(default=True)),
                ('email', models.EmailField(max_length=75)),
                ('text', models.TextField(max_length=512)),
                ('created_at', models.DateField()),
                ('updated_at', models.DateTimeField()),
                ('opened_at', models.TimeField()),
                ('percent', models.FloatField()),
                ('money', models.IntegerField()),
                ('ip', models.IPAddressField()),
                ('ip6', models.GenericIPAddressField(protocol='ipv6')),
                ('picture', models.FileField(upload_to=b'/tmp')),
                ('some_field', models.CommaSeparatedIntegerField(max_length=12)),
                ('funny', models.NullBooleanField()),
                ('slug', models.SlugField()),
                ('speed', models.DecimalField(max_digits=3, decimal_places=1)),
                ('url', models.URLField(default=b'', null=True, blank=True)),
                ('file_path', models.FilePathField()),
                ('object_id', models.PositiveIntegerField()),
                ('error_code', models.PositiveSmallIntegerField()),
                ('custom', CustomField(max_length=24)),
                ('content_type', models.ForeignKey(to='contenttypes.ContentType', on_delete=models.CASCADE)),
                ('binary', models.BinaryField()),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='Silk',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('color', models.CharField(max_length=20)),
                ('hat', models.ForeignKey(to='django_app.Hat', on_delete=models.CASCADE)),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='Simple',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('value', models.IntegerField()),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='Tag',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('title', models.CharField(max_length=20)),
                ('customer', models.ForeignKey(blank=True, to='django_app.Customer', on_delete=models.CASCADE, null=True)),
                ('messages', models.ManyToManyField(to='django_app.Message', null=True, blank=True)),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='Through',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('pointas', models.ForeignKey(to='django_app.PointA', on_delete=models.CASCADE)),
                ('pointbs', models.ForeignKey(to='django_app.PointB', on_delete=models.CASCADE)),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.AddField(
            model_name='rabbit',
            name='one2one',
            field=models.OneToOneField(to='django_app.Simple', on_delete=models.CASCADE),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='pointa',
            name='other',
            field=models.ManyToManyField(to='django_app.PointB', through='django_app.Through'),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='number',
            name='doors',
            field=models.ManyToManyField(to='django_app.Door'),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='number',
            name='wtf',
            field=models.ManyToManyField(related_name='wtf_rel_+', to='django_app.Number'),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='hole',
            name='owner',
            field=models.ForeignKey(to='django_app.Rabbit', on_delete=models.CASCADE),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='hat',
            name='owner',
            field=models.ForeignKey(blank=True, to='django_app.Rabbit', on_delete=models.CASCADE, null=True),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='door',
            name='hole',
            field=models.ForeignKey(to='django_app.Hole', on_delete=models.CASCADE),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='door',
            name='owner',
            field=models.ForeignKey(blank=True, to='django_app.Rabbit', on_delete=models.CASCADE, null=True),
            preserve_default=True,
        ),
    ]




""" Test mixer base functionality. """
import datetime

import pytest
from decimal import Decimal

from mixer.main import Mixer, TypeMixer


class Test:

    """ Model scheme for base tests. """

    one = int
    two = int
    name = str
    title = str
    body = str
    price = Decimal
    choices = list
    parts = set
    scheme = dict


def test_factory():
    """ Test base generator's factory. """
    from mixer.main import GenFactory

    g = GenFactory()
    test = g.get_fabric(int)
    assert -2147483647 <= test() < 2147483647

    test = g.get_fabric(bool)
    assert test() in (True, False)


def test_typemixer_meta():
    """ Tests that typemixer is a singleton for current class. """
    mixer1 = TypeMixer(Test)
    mixer2 = TypeMixer(Test, fake=False)
    mixer3 = TypeMixer(Test, fake=False)

    assert mixer1 is not mixer2
    assert mixer2 is mixer3


def test_typemixer():

    class Scheme:
        id = int
        name = str
        money = int
        male = bool
        prop = Test

    mixer = TypeMixer(Scheme)
    test = mixer.blend(prop__two=2, prop__one=1, prop__name='sigil', name='RJ')
    assert test.male in (True, False)
    assert test.name == 'RJ'
    assert test.prop.two == 2
    assert test.prop.name == 'sigil'

    test = mixer.blend(prop__two=4, unknown=lambda: '?')
    assert test.prop.two == 4
    assert test.unknown == '?'


def test_fake():
    from mixer.main import mixer

    test = mixer.blend(Test, name=mixer.FAKE, title=mixer.FAKE)
    assert ' ' in test.name
    assert ' ' in test.title

    test = mixer.blend(Test, name=mixer.FAKE(bool))
    assert test.name in (True, False)


def test_random():
    from mixer._compat import string_types

    mixer = TypeMixer(Test)
    test = mixer.blend(name=mixer.RANDOM)
    assert isinstance(test.name, string_types)

    test = mixer.blend(name=mixer.RANDOM(int))
    assert isinstance(test.name, int)

    names = ['john_', 'kenn_', 'lenny_']
    test = mixer.blend(name=mixer.RANDOM(*names))
    assert test.name in names


def test_mix():
    from mixer.main import mixer

    lama = type('One', tuple(), dict(
        two=int,
        one=type('Two', tuple(), dict(two=2.1))
    ))
    mix = mixer.MIX.one.two
    assert mix & lama == 2.1

    test = mixer.blend(lama, one__two=2.1)
    assert test.one.two == 2.1
    assert test.two != test.one.two

    test = mixer.blend(lama, one__two=2.1, two=mixer.MIX.one.two)
    assert test.two == test.one.two


def test_mixer():
    mixer = Mixer()

    assert Mixer.SKIP == mixer.SKIP
    try:
        Mixer.SKIP = 111
        raise AssertionError('test are failed')
    except AttributeError:
        pass
    try:
        mixer.SKIP = 111
        raise AssertionError('test are failed')
    except AttributeError:
        pass

    gen = ('test{0}'.format(i) for i in range(500))
    test = mixer.blend('tests.test_main.Test', name=gen)
    assert test.name == 'test0'

    name_gen = mixer.sequence(lambda c: 'test' + str(c))
    test = mixer.blend(Test, name=name_gen)
    test = mixer.blend(Test, name=name_gen)
    test = mixer.blend(Test, name=name_gen)
    assert test.name == 'test2'

    name_gen = mixer.sequence('test{0}')
    test = mixer.blend(Test, name=name_gen)
    test = mixer.blend(Test, name=name_gen)
    assert test.name == 'test1'

    name_gen = mixer.sequence()
    test = mixer.blend(Test, name=name_gen)
    test = mixer.blend(Test, name=name_gen)
    assert test.name == 1

    mixer.register('tests.test_main.Test',
                   name='Michel', one=lambda: 'ID', unknown="No error here")
    test = mixer.blend(Test)
    assert test.one == 'ID'
    assert test.name == 'Michel'


def test_mixer_cycle():
    mixer = Mixer()
    test = mixer.cycle(3).blend(Test)
    assert len(test) == 3
    assert test[0].__class__ == Test

    test = mixer.cycle(3).blend(Test, name=mixer.sequence('lama{0}'))
    assert test[2].name == 'lama2'


def test_mixer_default():
    from mixer.main import mixer

    test = mixer.blend(Test)
    assert test.name


def test_invalid_scheme():
    from mixer.main import mixer

    with pytest.raises(ValueError):
        mixer.blend('tests.test_main.Unknown')


def test_sequence():
    from mixer.main import mixer

    gen = mixer.sequence()
    assert next(gen) == 0
    assert next(gen) == 1

    gen = mixer.sequence('test - {0}')
    assert next(gen) == 'test - 0'
    assert next(gen) == 'test - 1'

    gen = mixer.sequence(lambda c: c + 2)
    assert next(gen) == 2
    assert next(gen) == 3

    gen = mixer.sequence(4, 3)
    assert next(gen) == 4
    assert next(gen) == 3
    assert next(gen) == 4


def test_custom():
    mixer = Mixer()

    @mixer.middleware(Test)
    def postprocess(x): # noqa
        x.name += ' Done'
        return x

    mixer.register(
        Test,
        name='Mike',
        one=mixer.faker.pyfloat,
        body=mixer.faker.date_time,
    )

    test = mixer.blend(Test)
    assert test.name == 'Mike Done'
    assert isinstance(test.one, float)
    assert isinstance(test.body, datetime.datetime)

    from mixer.main import GenFactory

    class MyFactory(GenFactory):
        generators = {str: lambda: "Always same"}

    mixer = Mixer(factory=MyFactory, fake=False)
    test = mixer.blend(Test)
    assert test.name == "Always same"


def test_ctx():
    from mixer.main import LOGGER

    mixer = Mixer()
    level = LOGGER.level

    with mixer.ctx(loglevel='INFO'):
        mixer.blend(Test)
        assert LOGGER.level != level

    with mixer.ctx(locale='ru'):
        dw = mixer.faker.day_of_week()
        assert dw[0] in 'ПВСЧ'

    assert LOGGER.level == level


def test_locale():
    mixer = Mixer()
    mixer.faker.locale = 'ru'

    with mixer.ctx(locale='it'):
        mixer.faker.name()

    assert mixer.faker.locale == 'ru_RU'

    with mixer.ctx(loglevel='INFO'):
        mixer.faker.name()

    assert mixer.faker.locale == 'ru_RU'


def test_silence():
    mixer = Mixer()

    class CustomException(Exception):
        pass

    @mixer.middleware(Test)
    def falsed(test): # noqa
        raise CustomException('Unhandled')

    with pytest.raises(CustomException):
        mixer.blend(Test)

    with mixer.ctx(silence=True):
        mixer.blend(Test)

    mixer.unregister_middleware(Test, falsed)
    mixer.blend(Test)  # does not raise any exceptions


def test_guard():
    mixer = Mixer()
    test = mixer.guard().blend(Test)
    assert test


def test_skip():
    mixer = Mixer()
    test = mixer.blend(Test, one=mixer.SKIP)
    assert test.one is not mixer.SKIP
    assert test.one is int


def test_reload():
    mixer = Mixer()
    test = mixer.blend(Test)
    test2 = mixer.reload(test)
    assert test is not test2
    assert test.name == test2.name

    test3, test4 = mixer.reload(test, test2)
    assert test3 and test4



from decimal import Decimal
from datetime import datetime

from pony import orm

db = orm.Database("sqlite", ":memory:", create_db=True)


class Customer(db.Entity):
    address = orm.Required(str)
    country = orm.Required(str)
    email = orm.Required(str, unique=True)
    name = orm.Required(str)
    password = orm.Required(str)

    cart_items = orm.Set("CartItem")
    orders = orm.Set("Order")


class Product(db.Entity):
    id = orm.PrimaryKey(int, auto=True)
    name = orm.Required(str)
    categories = orm.Set("Category")
    description = orm.Optional(str)
    picture = orm.Optional(bytes)
    price = orm.Required(Decimal)
    quantity = orm.Required(int)
    cart_items = orm.Set("CartItem")
    order_items = orm.Set("OrderItem")


class CartItem(db.Entity):
    quantity = orm.Required(int)
    customer = orm.Required(Customer)
    product = orm.Required(Product)


class OrderItem(db.Entity):
    quantity = orm.Required(int, default=1)
    price = orm.Required(Decimal)
    order = orm.Required("Order")
    product = orm.Required(Product)
    orm.PrimaryKey(order, product)


class Order(db.Entity):
    id = orm.PrimaryKey(int, auto=True)
    state = orm.Required(str)
    date_created = orm.Required(datetime)
    date_shipped = orm.Optional(datetime)
    date_delivered = orm.Optional(datetime)
    total_price = orm.Required(Decimal)
    customer = orm.Required(Customer)
    items = orm.Set(OrderItem)


class Category(db.Entity):
    name = orm.Required(str, unique=True)
    products = orm.Set(Product)


db.generate_mapping(create_tables=True)


def test_backend():
    from mixer.backend.pony import mixer
    assert mixer


@orm.db_session
def test_mixer():
    from mixer.backend.pony import mixer

    customer = mixer.blend(Customer)
    assert customer.name
    assert customer.email

    product = mixer.blend(Product)
    assert product.price

    order = mixer.blend(Order)
    assert order.customer

    orderitem = mixer.blend(OrderItem, product=product)
    assert orderitem.quantity == 1
    assert orderitem.order

    order = mixer.blend(Order, customer__name='John Snow')
    assert order.customer.name == 'John Snow'

    with mixer.ctx(commit=True):
        order = mixer.blend(Order)
        assert order.id
